#!/usr/bin/env python

import os
import sys

from django.conf import settings
from django.core.management import execute_from_command_line


if not settings.configured:
    params = dict(
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
            }
        },
        INSTALLED_APPS=[
            'django.contrib.contenttypes',
            'django.contrib.auth',
            'django.contrib.sites',
            'wagtail.wagtailcore',
            'wagtail.wagtailsites',
            'wagtail.wagtailusers',
            'wagtail.wagtailimages',
            'taggit',
            "wagtailaltgenerator",
            "tests",
        ],
        MIDDLEWARE_CLASSES=[],
        ROOT_URLCONF='tests.urls',
        ALT_GENERATOR_MIN_CONFIDENCE=0,
    )

    if 'COMPUTER_VISION_API_KEY' in os.environ:
        params.update(dict(
            COMPUTER_VISION_API_KEY=os.environ['COMPUTER_VISION_API_KEY'],
        ))

    settings.configure(**params)


def runtests():
    argv = sys.argv[:1] + ["test"] + sys.argv[1:]
    execute_from_command_line(argv)


if __name__ == "__main__":
    runtests()



import requests
import os


def get_image_data(image_url):
    '''
    Load external image and return byte data
    '''
    image_data = requests.get(image_url)

    if image_data.status_code > 200 and image_data.status_code < 300:
        return None

    return image_data.content


def get_local_image_data(image_file):
    '''
    Retrive byte data from a local file
    '''
    abs_path = os.path.abspath(image_file.path)
    image_data = open(abs_path).read()
    return image_data



from django.conf import settings


DEFAULT_PROVIDER = 'wagtailaltgenerator.providers.cognitive.Cognitive'  # NOQA

ALT_GENERATOR_USE_TAGS = getattr(settings, 'ALT_GENERATOR_USE_TAGS', True)
ALT_GENERATOR_MAX_TAGS = getattr(settings, 'ALT_GENERATOR_MAX_TAGS', -1)
ALT_GENERATOR_PROVIDER = getattr(settings,
                                 'ALT_GENERATOR_PROVIDER', DEFAULT_PROVIDER)
ALT_GENERATOR_MIN_CONFIDENCE = getattr(settings,
                                       'ALT_GENERATOR_MIN_CONFIDENCE', 50)
ALT_GENERATOR_PREFER_UPLOAD = getattr(settings, 'ALT_GENERATOR_PREFER_UPLOAD',
                                      True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.apps import AppConfig


class AltGeneratorAppConfig(AppConfig):
    name = 'wagtailaltgenerator'
    verbose_name = 'Alt generator'

    def ready(self):
        import wagtailaltgenerator.signals  # NOQA



import importlib

from wagtailaltgenerator import app_settings


def get_current_provider():
    provider = app_settings.ALT_GENERATOR_PROVIDER
    return get_provider(provider)


def get_provider(path):
    module_name, class_name = path.rsplit('.', 1)
    return getattr(importlib.import_module(module_name), class_name)


class AbstractProvider(object):
    def describe(image):
        pass


class DescriptionResult(object):
    def __init__(self, description, tags):
        self.description = description
        self.tags = tags



# -*- coding: utf-8 -*-
import logging

from django.conf import settings
import requests

from wagtailaltgenerator import app_settings
from wagtailaltgenerator.providers import (
    AbstractProvider,
    DescriptionResult,
)
from wagtailaltgenerator.utils import (
    get_image_data,
    get_local_image_data,
)


API_URL = 'https://api.projectoxford.ai'

logger = logging.getLogger(__name__)


def describe_by_url(image_url):
    headers = {
        'Content-Type': 'application/json',
        'Ocp-Apim-Subscription-Key': settings.COMPUTER_VISION_API_KEY,
    }

    json_data = {
        "url": image_url,
    }

    response = requests.post('{}{}'.format(API_URL, '/vision/v1.0/describe'),
                             headers=headers,
                             json=json_data
                             )

    if response.status_code != 200:
        logging.warn(response)
        return None

    return response.json()


def describe_by_data(image_data):
    headers = {
        'Content-Type': 'application/octet-stream',
        'Ocp-Apim-Subscription-Key': settings.COMPUTER_VISION_API_KEY,
    }

    response = requests.post('{}{}'.format(API_URL, '/vision/v1.0/describe'),
                             data=image_data,
                             headers=headers,
                             )

    if response.status_code != 200:
        logging.warn(response)
        return None

    return response.json()


class Cognitive(AbstractProvider):
    def describe(self, image):
        if app_settings.ALT_GENERATOR_PREFER_UPLOAD:
            if not image.is_stored_locally():
                image_data = get_image_data(image.file.url)
                data = describe_by_data(image_data)
            else:
                image_data = get_local_image_data(image.file)
                data = describe_by_data(image_data)
        else:
            data = describe_by_url(image.file.url)

        description = None
        tags = []

        min_confidence = float(app_settings.ALT_GENERATOR_MIN_CONFIDENCE)/100.0

        if not data:
            return DescriptionResult(
                description=description,
                tags=tags,
            )

        if 'description' in data and len(data['description']['captions']):
            captions = data['description']['captions']
            captions = [caption['text'] for caption in captions
                        if caption['confidence'] >= min_confidence]

            if len(captions):
                description = captions[0]

        try:
            tags = data['description']['tags']
        except:
            pass

        return DescriptionResult(
            description=description,
            tags=tags,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
wagtailaltgenerator
----------
Insert image description and tags with the help of computer vision
"""

__title__ = "wagtailaltgenerator"
__version__ = '3.1.0'
__build__ = 310
__author__ = "Martin Sandström"
__license__ = "MIT"
__copyright__ = "Copyright 2016 Martin Sandström"

default_app_config = 'wagtailaltgenerator.apps.AltGeneratorAppConfig'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

from django.db.models.signals import post_save
from django.dispatch import receiver

try:
    from wagtail.wagtailimages import get_image_model
except ImportError:
    from wagtail.wagtailimages.models import get_image_model

from wagtailaltgenerator.providers import get_current_provider
from wagtailaltgenerator import app_settings

image_cls = get_image_model()


@receiver(post_save, sender=image_cls, dispatch_uid="apply_image_alt")
def apply_image_alt(sender, instance, **kwargs):
    if not kwargs['created']:
        return

    provider = get_current_provider()()
    image_url = instance.file.url

    result = provider.describe(instance)

    if image_url[-4:] == instance.title[-4:]:
        _apply_title(instance, result)

    if app_settings.ALT_GENERATOR_USE_TAGS:
        _apply_tags(instance, result)

    instance.save()


def _apply_title(instance, result):
    if not result.description:
        return

    instance.title = result.description


def _apply_tags(instance, result):
    if not result.tags:
        return

    tags = result.tags

    if app_settings.ALT_GENERATOR_MAX_TAGS != -1:
        tags = tags[:app_settings.ALT_GENERATOR_MAX_TAGS]

    instance.tags.add(*tags)



# -*- coding: utf-8 -*-
import base64

from googleapiclient import discovery
from oauth2client.client import GoogleCredentials

from wagtailaltgenerator import app_settings
from wagtailaltgenerator.providers import (
    AbstractProvider,
    DescriptionResult
)
from wagtailaltgenerator.utils import (
    get_image_data,
    get_local_image_data,
)


class GoogleVision(AbstractProvider):
    def __init__(self, *args, **kwargs):
        credentials = GoogleCredentials.get_application_default()
        self.service = discovery.build('vision', 'v1', credentials=credentials)
        super(GoogleVision, self).__init__(*args, **kwargs)

    def describe(self, image):
        if not image.is_stored_locally():
            image_data = get_image_data(image.file.url)
        else:
            image_data = get_local_image_data(image.file)

        description = None
        tags = []

        min_confidence = float(app_settings.ALT_GENERATOR_MIN_CONFIDENCE)/100.0

        if image_data:
            image_data_base64 = base64.b64encode(image_data)

            max_results = app_settings.ALT_GENERATOR_MAX_TAGS

            request_body = {
                'requests': [{
                    'image': {
                        'content': image_data_base64.decode('UTF-8'),
                    },
                    'features': [{
                        'type': 'LABEL_DETECTION',
                    }]
                }]
            }

            if max_results != -1:
                request_body['requests'][0]['features'][0]['maxResults'] = max_results  # NOQA

            service_request = self.service.images().annotate(body=request_body)

            response = service_request.execute()
            labels = response['responses'][0]['labelAnnotations']
            tags = [label['description'] for label in labels
                    if label['score'] >= min_confidence]

        return DescriptionResult(
            description=description,
            tags=tags,
        )



# -*- coding: utf-8 -*-

import boto3

from wagtailaltgenerator import app_settings
from wagtailaltgenerator.providers import (
    AbstractProvider,
    DescriptionResult
)
from wagtailaltgenerator.utils import (
    get_image_data,
    get_local_image_data,
)


class Rekognition(AbstractProvider):
    def __init__(self, *args, **kwargs):
        self.client = boto3.client('rekognition')
        super(Rekognition, self).__init__(*args, **kwargs)

    def describe(self, image):
        if not image.is_stored_locally():
            image_data = get_image_data(image.file.url)
        else:
            image_data = get_local_image_data(image.file)

        description = None
        tags = []

        if image_data:
            response = self.client.detect_labels(
                Image={
                    'Bytes': image_data,
                },
                MinConfidence=app_settings.ALT_GENERATOR_MIN_CONFIDENCE,
            )
            if response['Labels']:
                tags = [label['Name'] for label in response['Labels']]

        return DescriptionResult(
            description=description,
            tags=tags,
        )
